#!/usr/bin/env python
# -*- coding:utf-8 -*-
from datetime import datetime

import pandas as pd

from braincube.bc_connector.braincube_requests import get_references, get_data_defs
from braincube.bc_connector.data_formatting import data_to_dataframe
from braincube.bc_connector.raw_retriever import RawRetriever


class PandasRetriever(RawRetriever):  # Class giving access to functions communicating with the braincube API
    def __init__(self, sso_token, braincubes):
        RawRetriever.__init__(self, sso_token, braincubes)
        self.__sso_token = sso_token
        self.__braincubes = braincubes

    def get_braincube_list(self):
        # Return a dataframe containing the braincubes accessible
        try:
            bc_id = []
            bc_name = []

            for i in self.__braincubes:
                if i['product']['type'] == 'braincube':
                    bc_id.append(i['product']['productId'])
                    bc_name.append(i['product']['name'])
            return pd.DataFrame({'product_id': bc_id, 'name': bc_name})
        except TypeError:
            print("No data retrieved")
        return

    def get_memorybase_list(self, braincube_name):
        # Return a dataframe containing the memorybases available for the braincube selected
        try:
            mb_name = []
            mb_nb_var = []
            mb_id = []
            print('Braincube selected : ' + braincube_name)
            memorybases = super(PandasRetriever, self).get_memorybase_list(braincube_name)
            for i in memorybases:
                if not i['quickStudy']:
                    mb_id.append(i['bcId'])
                    mb_name.append(i['name'])
                    mb_nb_var.append(i['numberOfVariables'])
            return pd.DataFrame({'id': mb_id, 'name': mb_name, 'number_of_variable': mb_nb_var})
        except TypeError:
            print("No data retrieved")

    def get_memorybase_order_variable(self, braincube_name, mb_id):
        # Return the id of the variable which is used to order the memorybase
        ref = super(PandasRetriever, self).get_memorybase_order_variable(braincube_name, mb_id)
        return ref['referenceDate'].split('/d')[1]

    def get_variable_list(self, braincube_name, mb_id):
        # Return a dataframe containing the variables available for the memorybase selected
        try:
            var_name = []
            var_id = []
            data_def = super(PandasRetriever, self).get_variable_list(braincube_name, mb_id)
            for i in data_def:
                var_id.append(i['id'])
                var_name.append(i['local'])
            return pd.DataFrame({'id': var_id, 'name': var_name})
        except TypeError:
            print("No data retrieved")

    def retrieve_all_variables_from_memory_base(self, braincube_name, mb_id, start_date, end_date=datetime.now()):
        # Return a dataframe containing the datas of all the variables of the selected memorybase
        # Indexed with the order variable, a column represents a variable
        variable_list = self.get_variable_list(braincube_name, mb_id)
        selected_variables = []
        for var in variable_list.itertuples():
            selected_variables.append(var.id)
        return self.retrieve_data(braincube_name, mb_id, selected_variables, start_date, end_date)

    def retrieve_data(self, braincube_name, mb_id, variable_list, start_date, end_date=datetime.now()):
        # Return a dataframe containing the datas of the seelcted variables for the selected memorybase
        # Indexed with the order variable, a column represents a variable
        try:
            ref = get_references(braincube_name, mb_id, self.__sso_token)
            data = super(PandasRetriever, self) \
                .retrieve_data(braincube_name, mb_id, variable_list, start_date, end_date)
            print('Format the data')
            data_def = get_data_defs(braincube_name, mb_id, self.__sso_token)
            return data_to_dataframe(data, data_def, ref)
        except TypeError:
            print("No data retrieved")



import random
from os.path import exists, join
from socket import gethostname

from OpenSSL import crypto

CERT_FILE = "cert.pem"
KEY_FILE = "key.pem"


def gen_ssl_cert_if_none(config_dir):
    """
    If cert.pem and key.pem are not found in ~/.braincube, generate them (self-signed certificate)
    """
    if not exists(join(config_dir, CERT_FILE)) \
            or not exists(join(config_dir, KEY_FILE)):
        # create a key pair
        key = crypto.PKey()
        key.generate_key(crypto.TYPE_RSA, 2048)
        # create a self-signed cert
        cert = crypto.X509()
        cert.get_subject().C = "FR"  # country code
        cert.get_subject().L = "Issoire"  # locality
        cert.get_subject().OU = "Braincube"  # organisation
        cert.get_subject().CN = gethostname()
        cert.set_serial_number(random.getrandbits(128))
        cert.gmtime_adj_notBefore(0)
        cert.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)
        cert.set_issuer(cert.get_subject())
        cert.set_pubkey(key)
        print("Generate self-signed ssl certificate.")
        cert.sign(key, 'sha256')
        open(join(config_dir, CERT_FILE), "wb").write(
            crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
        open(join(config_dir, KEY_FILE), "wb").write(
            crypto.dump_privatekey(crypto.FILETYPE_PEM, key))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import json
import traceback

import requests
import logging

__sso_token_uri = "https://mybraincube.com/sso-server/rest/session/openWithToken"


def get_sso_token(oauth2_acces_token):
    # Return a sso_token thanks to the token, which allows the access to the REST API
    if str(oauth2_acces_token):
        headers = {'Authorization': 'Bearer ' + str(oauth2_acces_token)}

        try:
            result = requests.get(__sso_token_uri, headers=headers)
            if result.status_code != 200:
                print("Authentication failure: try again with other credentials")
            else:
                print("Successful authentication, you can now access your data")
                return json.loads(result.text)
        except Exception as e:
            logging.debug(traceback.format_exc())


def request_data(ref, selected_variables, braincube_name, start_date, end_date, sso_token):
    # Return the result of the request of data retrieving, construct with the params of the user
    if ref['referenceDate'] not in selected_variables:
        selected_variables.append(ref['referenceDate'])
    url = "https://api.mybraincube.com/braincube/" \
          + braincube_name + "/braindata/" \
          + ref['name'] + "/LF"
    headers = {'Content-Type': 'application/json', 'Accept': "application/json", 'IPLSSOTOKEN': sso_token}
    filters = [ref['order'], start_date, end_date]
    content = {
        "order": ref['referenceDate'],
        "definitions": selected_variables,
        "context":
            {
                "dataSource": ref['name'],
                "filter":
                    {
                        "BETWEEN": filters
                    }
            }
    }
    result = requests.post(url, data=json.dumps(content), headers=headers)
    return json.loads(result.text)


def get_references(braincube_name, mb_id, sso_token):
    # Return the reference of the memorybase selected
    ref_url = "https://api.mybraincube.com/braincube/" + braincube_name + "/braindata/mb" + str(mb_id) + "/simple"
    ref_headers = {'IPLSSOTOKEN': sso_token}
    res = requests.get(ref_url, headers=ref_headers)
    return json.loads(res.text)


def get_data_defs(braincube_name, mb_id, sso_token):
    # Return the definitions of the variables for the selected memorybase
    url = "https://api.mybraincube.com/braincube/" + braincube_name + "/braincube/mb/" \
          + str(mb_id) + "/variables/selector"
    headers = {'IPLSSOTOKEN': sso_token}
    result = requests.get(url, headers=headers)
    # print(json.loads(result.text))
    return json.loads(result.text)['items']


def get_memorybase(braincube_name, sso_token):
    # Return the memorybases available for the selected braincube
    url = "https://api.mybraincube.com/braincube/" + braincube_name + "/braincube/mb/all/selector"
    headers = {'IPLSSOTOKEN': sso_token}
    result = requests.get(url, headers=headers)
    # print(json.loads(result.text))
    return json.loads(result.text)['items']




# class TestConfigHandler(TestCase):
from braincube.bc_connector.config_handler import is_config_filled


def test_is_config_filled_ko_01():
    assert is_config_filled({"client_secret": "", "client_id": ""}) is False


def test_is_config_filled_ko_02():
    assert is_config_filled({"client_secret": "", "client_id": "test"}) is False


def test_is_config_filled_ko_03():
    assert is_config_filled({"client_secret": "test", "client_id": ""}) is False


def test_is_config_filled_ok():
    assert is_config_filled({"client_secret": "test", "client_id": "test"}) is True



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import json
from datetime import datetime

from braincube.bc_connector.braincube_requests import get_data_defs, get_memorybase, get_references, request_data
from braincube.bc_connector.entities.braincube import Braincube
from braincube.bc_connector.entities.memorybase import Memorybase


class RawRetriever:  # Class giving access to functions communicating with the braincube API
    def __init__(self, sso_token, braincubes):
        self.__sso_token = sso_token
        self.__braincubes = braincubes

    def get_braincube_list(self):
        # Return a dataframe containing the braincubes accessible
        return self.__braincubes

    def get_memorybase_list(self, braincube_name):
        # Return a dataframe containing the memorybases available for the braincube selected
        try:
            return get_memorybase(braincube_name, self.__sso_token)
        except json.decoder.JSONDecodeError:
            print("Invalid parameters, please check spelling or access rights")

    def get_memorybase_order_variable(self, braincube_name, mb_id):
        # Return the id of the variable which is used to order the memorybase
        try:
            return get_references(braincube_name, mb_id, self.__sso_token)
        except json.decoder.JSONDecodeError:
            print("Invalid parameters, please check spelling or access rights")

    def get_variable_list(self, braincube_name, mb_id):
        # Return a dataframe containing the variables available for the memorybase selected
        try:
            return get_data_defs(braincube_name, mb_id, self.__sso_token)
        except json.decoder.JSONDecodeError:
            print("Invalid parameters, please check spelling or access rights")

    def retrieve_all_variables_from_memory_base(self, braincube_name, mb_id, start_date, end_date=datetime.now()):
        # Return a dataframe containing the datas of all the variables of the selected memorybase
        # Indexed with the order variable, a column represents a variable
        variable_list = self.get_variable_list(braincube_name, mb_id)
        selected_variables = []
        for var in variable_list:
            selected_variables.append(var['id'])
        return self.retrieve_data(braincube_name, mb_id, selected_variables, start_date, end_date)

    def retrieve_data(self, braincube_name, mb_id, variable_list, start_date, end_date=datetime.now()):
        # Return a dataframe containing the datas of the seelcted variables for the selected memorybase
        # Indexed with the order variable, a column represents a variable
        try:
            print('Braincube selected : ' + braincube_name)
            print('MemoryBase selected : ' + str(mb_id))
            ref = get_references(braincube_name, mb_id, self.__sso_token)
            selected_variables = []
            for var in variable_list:
                selected_variables.append(ref['name'] + '/d' + str(var))
            return request_data(ref, selected_variables,
                                braincube_name, start_date.strftime("%Y%m%d_%H%M%S"),
                                end_date.strftime("%Y%m%d_%H%M%S"), self.__sso_token)['datadefs']
        except json.decoder.JSONDecodeError:
            print("Invalid parameters, please check spelling or access rights")

    def get_braincube(self, braincube_name):
        return Braincube(braincube_name, self)

    def get_memorybase(self, braincube_name, mb_id):
        return Memorybase(braincube_name, mb_id, self)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import pandas as pd
from datetime import datetime


def get_data_with_id(data):
    # Transforms the data into a dictionary with the variable's id and the corresponding data for that variable
    var = {}
    for i in data:
        var[i['id'].split('/d')[1]] = i['data']
    return var


def data_to_dataframe(datas, datadefs, ref):
    # Return a dataframe based on the given data
    data = get_data_with_id(datas)
    index = []
    result = {}
    for i in datadefs:
        if str(i['id']) in data.keys():
            if str(i['id']) == ref['referenceDate'].split('/d')[1]:
                for j in data[str(i['id'])]:  # Changes the format of the date to be readable
                    data[str(i['id'])][data[str(i['id'])].index(j)] = datetime.strptime(j, "%Y%m%d_%H%M%S") \
                        .strftime("%Y/%m/%d_%H:%M:%S")
                index = data[str(i['id'])]
            else:
                result[i['local']] = data[str(i['id'])]
    dataframe = pd.DataFrame(result, index=index)
    print("Your data has been recovered")
    return dataframe



import json as __json
import os
import re as __research
import webbrowser

import requests
from flask import Flask as __Flask, send_from_directory, render_template
from flask import redirect
from flask import request
from flask_cors import CORS as __CORS
from flask_cors import cross_origin
from gevent import pywsgi

from braincube.bc_connector.braincube_requests import get_sso_token
from braincube.bc_connector.config_handler import get_client_id, get_client_secret

file_path = os.path.expanduser('~/.braincube') + '/OAuth2AccessToken'

__app = __Flask(__name__)
__app.config['CORS_HEADERS'] = 'Content-Type'
__cors = __CORS(__app, resources={r"/*": {"origins": "https://mybraincube.com/*"}})

cert_file = os.path.expanduser('~/.braincube/cert.pem')
key_file = os.path.expanduser('~/.braincube/key.pem')

http_server = pywsgi.WSGIServer(('localhost', 5000), __app, certfile=cert_file, keyfile=key_file)

authorize_uri = "https://mybraincube.com/sso-server/vendors/braincube/authorize.jsp"
token_uri = "https://mybraincube.com/sso-server/ws/oauth2/token"
redirect_uri = "https://localhost:5000/token"

scopes = "BASE%20API"


@__app.route('/')
@cross_origin()
# Code executed when launching the local server
# Opens the Braincube authorization page or the React page according to the parameter passed in query
def launch_connector():
    url = authorize_uri \
          + '?client_id=' + get_client_id() \
          + '&response_type=code' \
          + '&scope=' + scopes \
          + '&redirect_uri=' + redirect_uri
    print("url " + url)
    return redirect(url)  # -> /token


@__app.route('/token')
@cross_origin()
# Code executed when returning from the Braincube authorization page
# Allows to recover a token thanks to the received code
def get_token_from_code():
    code = __research.search('code=(.*)', request.url).group(1)
    content = {"grant_type": "authorization_code",
               "code": code,
               "redirect_uri": redirect_uri,
               "client_id": get_client_id(),
               "client_secret": get_client_secret()
               }
    print("Pending recovery attempt, please wait")
    retrieve_token = requests.post(token_uri, data=content)
    global TOKEN  # The only way to return a result from a flask app
    TOKEN = __json.loads(retrieve_token.text)['access_token']
    return render_template("closing.html")


@__app.route('/closing')
@cross_origin()
def closing():
    shutdown_server()
    return "server closed"


def shutdown_server():
    """ Close the local server"""
    print("Shutting down the local serveur.")
    http_server.close()


def retrieve_tokens(oauth2_acces_token=None):
    """ Try to get an sso token from an OAuth2 access token"""
    sso_token = None
    if oauth2_acces_token is not None:  # if we have a specified token
        print("Trying with specified oauth2 access token")
        sso_token = get_sso_token(oauth2_acces_token)
    if sso_token is None:
        print("Trying with locally saved oauth2 access token")
        oauth2_acces_token = read_local_oauth_access_token()  # Try to read local written token
        sso_token = get_sso_token(oauth2_acces_token)
    if sso_token is None:
        print("Trying by calling SSO to acquire an oauth2 access token ")
        oauth2_acces_token = get_token_from_web()  # Retrieve a token from web
        sso_token = get_sso_token(oauth2_acces_token)
    return oauth2_acces_token, sso_token


def get_token_from_web():
    """ Redirect to the sso server to get an oauth2 token """
    webbrowser.open('https://localhost:5000/')
    try:
        http_server.serve_forever()

    except OSError:
        print('The port 5000 of the machine is already open, please close your activity before restarting it')
    token = TOKEN
    write_local_oauth_access_token(token)
    return token


def read_local_oauth_access_token():
    """ Try to read the OAuth access token on disk """
    if os.path.isfile(file_path):
        with open(file_path) as token:
            return token.read()


def write_local_oauth_access_token(sso_token):
    """ write the OAuth access token on disk """
    os.makedirs(os.path.dirname(file_path), exist_ok=True)
    with open(file_path, 'w') as outfile:
        outfile.write(sso_token)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from braincube.bc_connector.config_handler import config_setup_ok
from braincube.bc_connector.pandas_retriever import PandasRetriever
from braincube.bc_connector.raw_retriever import RawRetriever


class Connector:
    def __init__(self, retriever, oauth2_access_token):
        self.__retriever = retriever
        self.__oauth2_access_token = oauth2_access_token

    def get_braincube_list(self):
        """
        :return: The list of available Braincube
        """
        return self.__retriever.get_braincube_list()

    def get_braincube(self, braincube_name):
        """
        :param braincube_name: a string with a braincube name
        :return: a braincube object, useful to access memorybases
        """
        return self.__retriever.get_braincube(braincube_name)

    def get_token(self):
        """
        :return: the oauth2 access token
        """
        return self.__oauth2_access_token


def get_data_collector(oauth2_acces_token=None, format_type='pandas'):
    """
    :param oauth2_acces_token: an OAuth2 access token can be specified. If none is set,
    :param format_type: 'raw' to return a connector working with plain python object, otherwise and by default return a connector working with pandas dataframe.
    :return: A connector allowing to access your braincube.
    """
    if config_setup_ok():
        # Function called by the user to access to flask_the python API
        # if the user call this function with a token, check it's validity and return an object object to manipulate the API
        print("Initializing braincube data collector...")
        from braincube.bc_connector.tokensHandler import retrieve_tokens
        oauth2_acces_token, sso_token = retrieve_tokens(oauth2_acces_token)

        if format_type == 'raw':
            retriever = RawRetriever(sso_token['token'], sso_token['accessList'])
        else:
            retriever = PandasRetriever(sso_token['token'], sso_token['accessList'])

        return Connector(retriever, oauth2_acces_token)
